module det_airy
use det_airy_types
use global_typesnvars
use ireg_airy, only: ireg_airy_pregen

implicit none

public :: det_airy_input
public :: det_airy_init
public :: det_airy_sea_elevation
public :: det_airy_get_kinematics

contains

Subroutine det_airy_input(masterfile,det)
use genout_tools
use global_typesnvars, only : wd

Type (Tmasterfile), pointer :: masterfile
Type (Tdet_airy),pointer		:: det

Type (Tline) :: line
Type (Tword) :: words

real*8, dimension(maxwords) :: rea

logical :: init, &
           out, &
           err, &
		   file_control, &
		   samples_control
!
init=.false.
out=.false.
file_control=.false.
samples_control=.false.
det%ireg%pregen = .FALSE.  ! pregen FALSE per default
!
do while (out==.false.)
  call readline(masterfile,line,init)
    call getwords(line%line,words,masterfile)
    select case (trim(words%word_vec(1)))
!    
	case ('file')
	  det%file = words%word_vec(2)
	  file_control = .true.
!
	case ('nsamples')
      call get_parameters(words,2,1,rea,err)
      if (err==.true.) then; goto 10; endif
	  det%nsamples = nint(rea(1))
	  samples_control = .true.
!
	case ('nskip')
      call get_parameters(words,2,1,rea,err)
      if (err==.true.) then; goto 10; endif
	  det%nskip = nint(rea(1))
!
	case ('columns')
      call get_parameters(words,2,2,rea,err)
      if (err==.true.) then; goto 10; endif
	  det%col_time = nint(rea(1))
	  det%col_elev = nint(rea(2))
!
	case ('stretching')
      call get_parameters(words,2,1,rea,err)
      if (err==.true.) then; goto 10; endif
      det%ireg%stretching=nint(rea(1))

	case ('cutoff_frac')
      call get_parameters(words,2,1,rea,err)
      if (err==.true.) then; goto 10; endif
      det%cutoff_frac = rea(1)
!
	case ('pregen')
	  call get_parameters(words,2,1,rea,err)
      if (err==.true.) then; goto 10; endif
	  if (nint(rea(1)).eq.1) then
	    det%ireg%pregen=.true.
	    allocate(det%ireg%wfield)
		write(0,*) 'In Wkin_dll: Wave field will be precalculated'
	  else
	    det%ireg%pregen=.false.
	  endif

    case ('')
      continue
!
    case ('end')
      out=.true.
      ! Control of data given above is OK
      if (file_control .and. samples_control) then
        write(0,*) 'Wkin dll input commands read with succes'
      else
        write(0,*) '*** ERROR *** Not all needed wkin_dll deterministic input commands present - error'
      endif
    case (';')
      continue
    case default
      write(0,*) '*** ERROR *** Error in command line ',masterfile%linenr,' Masterfile name: ',trim(masterfile%name)
    end select
enddo
return
10  write(0,*) '*** ERROR *** Error in command line ',masterfile%linenr,' Masterfile name: ',trim(masterfile%name)

end subroutine det_airy_input

!-------------------------------------------------------------

subroutine det_airy_init(det,t0,t1)
use numerical_libraries, only:dfftrf
use tools, only: newton_raphson
use reg_airy, only: dispersion, dispersion_deriv
! Input
Type (Tdet_airy) :: det
double precision :: t0,t1
! locals
real*8, pointer        :: wrec(:,:)
real*8, pointer        :: rea(:)
real*8, pointer        :: coef(:)
integer                :: nread      ! nof reals to read in each line
integer                :: nalloc
integer                :: unit = 10
integer                :: i,n
real*8                 :: tot_energy, acc_energy
integer                :: ilow, ihigh
real*8                 :: c_sin, c_cos
type (TCoef), pointer  :: c

!!DEC$ DEFINE WAVEDEBUG
!DEC$ IF DEFINED (WAVEDEBUG)
real*8 time, elev
interface
  subroutine set_new_time(time)
!DEC$ ATTRIBUTES ALIAS:'set_new_time'::set_new_time
  real*8 time
  end subroutine
end interface
!DEC$ ENDIF
!-----------------
! Read input file
!-----------------
nread = MAX(det%col_time, det%col_elev)   ! nof reals to read in each line
allocate(rea(nread))                      ! allocate buffer for read line
allocate(wrec(det%nsamples,2))            ! allocate time series
nalloc = (det%nsamples/2)*2 + 1           ! always odd
allocate(coef(nalloc))                    ! allocate fourier coefficients
! open file
open(unit,FILE=det%file)
! skip lines (given as input)
do i=1,det%nskip
  read(unit,*)
enddo
! read time series
do i=1,det%nsamples
  read(unit,*) rea
  wrec(i,1) = rea(det%col_time)
  wrec(i,2) = rea(det%col_elev)
enddo
!
close(unit)
!----------------------------------------------------------------------
! Fourier Transform
!----------------------------------------------------------------------
coef(nalloc) = 0.d0                      ! in case det%nsamples is even, last coef -> 0
CALL DFFTRF(det%nsamples, wrec(1,2), coef(1))
!----------------------------------------------------------------------
! Discard components with low energy
!----------------------------------------------------------------------
tot_energy = 0.d0
do i=1,(nalloc-1)/2
  n = i+1
  tot_energy = tot_energy + coef(2*n-2)**2 + coef(2*n-1)**2
enddo
! Discard low frequency energy
acc_energy = 0.d0
i=0
do while (acc_energy.lt. det%cutoff_frac*tot_energy)
  i = i+1
  n = i+1
  acc_energy = acc_energy + coef(2*n-2)**2 + coef(2*n-1)**2
enddo
ilow = i
! Discard high frequency energy
acc_energy = 0.d0
i = (nalloc-1)/2+1
do while (acc_energy.lt. det%cutoff_frac*tot_energy)
  i = i-1
  n = i+1
  acc_energy = acc_energy + coef(2*n-2)**2 + coef(2*n-1)**2
enddo
ihigh = i
!-----------------------
! Create wave components
!-----------------------
! NOF coefficients in truncated series
det%ireg%N = ihigh - ilow + 1
! Calc. period of total series (if not provided by input)
if (det%T.eq.0.d0) det%T = wrec(det%nsamples,1) - wrec(1,1)   ! T = t_last - t_first
! Calc. coefficients
allocate(det%ireg%coef(det%ireg%N))
do i=ilow,ihigh
  n = i+1
  c => det%ireg%coef(i-ilow+1)
!
  c_cos = 2.d0 * coef(2*n-2)
  c_sin = 2.d0 * coef(2*n-1)
!
  c%A     = SQRT(c_cos**2 + c_sin**2)/det%nsamples ! amplitude
  c%phi   = ATAN2(c_cos,-c_sin)                     ! phase
  c%omega = 2.d0*PI*(n-1)/det%T
  c%k     = newton_raphson(2.0d0,c%omega,wd%wdepth,dispersion,dispersion_deriv,1.0d-6)

!DEC$ IF DEFINED (WAVEDEBUG)
write(99,'(I,6(E15.6))') i-ilow+1, c_cos, c_sin, c%A, c%phi, c%omega, c%k
!DEC$ ENDIF
enddo
!
! Pre-generate if requested
!
if (det%ireg%pregen) then
  call ireg_airy_pregen(det%ireg,t0,t1,c%omega)  ! here, c%omega is the maximum frequency
endif

!
!DEC$ IF DEFINED (WAVEDEBUG)
do time = 0.d0, det%T, 0.2d0
  call set_new_time(time)
  call det_airy_sea_elevation((/0.d0,0.d0,0.d0/), det, elev)
  write(98,*) time, elev
enddo
!DEC$ ENDIF
  
end subroutine det_airy_init

!-------------------------------------------------------------

subroutine det_airy_sea_elevation(pos_h,det,elevation)
use ireg_airy, only: ireg_airy_sea_elevation
real*8,dimension(3)		:: pos_h
Type (Tdet_airy)		:: det
real*8					:: elevation	
!
call ireg_airy_sea_elevation(pos_h,det%ireg,elevation)
!
end subroutine det_airy_sea_elevation

!-------------------------------------------------------------

subroutine det_airy_get_kinematics(det,pos_h,vel_h,acc_h,pres)
use ireg_airy, only: ireg_airy_get_kinematics
Type (Tdet_airy)		:: det
real*8,dimension(3) :: pos_h
real*8,dimension(3) :: vel_h
real*8,dimension(3) :: acc_h
real*8				:: pres
!
call ireg_airy_get_kinematics(det%ireg, pos_h, vel_h, acc_h, pres)
!
end subroutine det_airy_get_kinematics
!
end module det_airy
