module reg_airy
use global_typesnvars

implicit none

public :: reg_airy_input
public :: reg_airy_init
public :: reg_airy_sea_elevation
public :: reg_airy_get_kinematics

public :: dispersion
public :: dispersion_deriv

contains

!-------------------------------------------------------------

Subroutine reg_airy_input(masterfile,reg)
use genout_tools
!use global_vars
use global_typesnvars, only : wd

Type (Tmasterfile), pointer :: masterfile
Type (Treg),pointer		:: reg

Type (Tline) :: line
Type (Tword) :: words

real*8, dimension(maxwords) :: rea

logical :: init, &
           out, &
           err, &
           all_ok, &
		   stretching_control, &
		   wave_control

init=.false.
out=.false.
stretching_control=.false.
wave_control=.false.

do while (out==.false.)
  call readline(masterfile,line,init)        ! line is read
    call getwords(line%line,words,masterfile)                           ! words in line is retrieved
    select case (trim(words%word_vec(1)))
    
	case ('stretching')
      call get_parameters(words,2,1,rea,err)
      if (err==.true.) then; goto 10; endif
      reg%stretching=nint(rea(1))
      stretching_control=.true.

	case ('wave')
      call get_parameters(words,2,2,rea,err)
      if (err==.true.) then; goto 10; endif
      reg%Hs=rea(1)
	  reg%A=rea(1)/2.d0
	  if (rea(2)>0.d0) then
        reg%omega=2.d0*pi/rea(2)
	  else
	    write(0,*) '*** ERROR *** In reg. airy input, T less or equal to 0.d0'
	  endif
      wave_control=.true.

    case ('')
      continue

    case ('end')
      out=.true.
      ! Control of data given above is OK
      all_ok=.true.
      if ((stretching_control==.false.) .or. &
	      (wave_control==.false.)) then
        all_ok=.false.
      endif
      if (all_ok==.true.) then
        write(0,*) 'Reg. airy input commands read with succes'
      else
        write(0,*) '*** ERROR *** Not all needed wkin_dll reg_airy input commands present - error'
      endif
    case (';')
      continue
    case default
      write(0,*) '*** ERROR *** Error in command line ',masterfile%linenr,' Masterfile name: ',trim(masterfile%name)
    end select
enddo
return
10  write(0,*) '*** ERROR *** Error in command line ',masterfile%linenr,' Masterfile name: ',trim(masterfile%name)

end subroutine reg_airy_input

!-------------------------------------------------------------

subroutine reg_airy_init(reg,t0,t1,dt)
use tools
use msimsl
Type (Treg)	:: reg
real*8			:: t0
real*8			:: t1
real*8			:: dt

! find wave number
reg%k=newton_raphson(2.0d0,reg%omega,wd%wdepth,dispersion,dispersion_deriv,1.0d-6)

end subroutine reg_airy_init

!-------------------------------------------------------------

subroutine reg_airy_sea_elevation(pos_h,reg,elevation)
real*8,dimension(3)		:: pos_h
Type (Treg)				:: reg
real*8					:: elevation	

elevation=reg%A*dsin(reg%omega*wd%time-pos_h(1)*reg%k)   ! elevation is positive upwards

end subroutine reg_airy_sea_elevation

!-------------------------------------------------------------

subroutine reg_airy_get_kinematics(reg,pos_h,vel_h,acc_h,pres)
Type (Treg)			:: reg
real*8,dimension(3) :: pos_h
real*8,dimension(3) :: vel_h
real*8,dimension(3) :: acc_h
real*8				:: pres

real*8				:: u,w
real*8				:: udot,wdot
real*8				:: elevation,zwheeler
Type (Tcoef),pointer	:: c
real*8				:: cosh1, cosh2, sinh1, sinh2, a_sin1, a_cos1

u=0.d0
w=0.d0
udot=0.d0
wdot=0.d0
pres=0.d0	
  select case (ireg%stretching)
  case (0)  ! no stretching
	a_sin1=reg%A*dsin(reg%omega*wd%time-reg%k*pos_h(1))
	a_cos1=reg%A*dcos(reg%omega*wd%time-reg%k*pos_h(1))

    elevation=a_sin1   ! elevation is positive upwards
    if (pos_h(3)<elevation) then
      cosh1=dcosh(c%k*(pos_h(3)+wd%wdepth))
	  sinh1=dsinh(c%k*(pos_h(3)+wd%wdepth))
	  sinh2=dsinh(c%k*wd%wdepth)
	  cosh2=dcosh(c%k*wd%wdepth)

      u=reg%omega*cosh1/sinh2*a_sin1
      w=reg%omega*sinh1/sinh2*a_cos1
      udot=reg%omega**2*cosh1/sinh2*a_cos1
      wdot=-reg%omega**2*sinh1/sinh2*a_sin1
      pres=wd%rho*wd%g*cosh1/cosh2*a_sin1
	endif
  case (1)	! wheeler stretching
	a_sin1=reg%A*dsin(reg%omega*wd%time-reg%k*pos_h(1))
	a_cos1=reg%A*dcos(reg%omega*wd%time-reg%k*pos_h(1))

    elevation=a_sin1   ! elevation is positive upwards
	if (pos_h(3)<elevation) then
      zwheeler=(pos_h(3)+wd%wdepth)/(1.0d0+elevation/wd%wdepth)

      cosh1=dcosh(reg%k*zwheeler)
	  sinh1=dsinh(reg%k*zwheeler)
	  sinh2=dsinh(reg%k*wd%wdepth)
	  cosh2=dcosh(reg%k*wd%wdepth)

      u=reg%omega*cosh1/sinh2*a_sin1
      w=reg%omega*sinh1/sinh2*a_cos1
      udot=reg%omega**2*cosh1/sinh2*a_cos1
      wdot=-reg%omega**2*sinh1/sinh2*a_sin1
      pres=wd%rho*wd%g*cosh1/cosh2*a_sin1
	endif
  case default
  end select

vel_h(1)=u
vel_h(2)=0.0d0
vel_h(3)=w

acc_h(1)=udot
acc_h(2)=0.0d0
acc_h(3)=wdot

end subroutine reg_airy_get_kinematics

!-------------------------------------------------------------

function dispersion(k,omega,d)
use global_typesnvars, only : wd
real*8		:: k		! wave number
real*8		:: omega	! frequency [rad/s]
real*8		:: d		! water depth
real*8		:: dispersion ! function output - target is zero

dispersion=wd%g*k*dtanh(k*d)-omega**2

end function dispersion

!-------------------------------------------------------------
function dispersion_deriv(k,omega)
use global_typesnvars, only : wd
real*8		:: k		! wave number
real*8		:: omega	! frequency [rad/s]
real*8		:: dispersion_deriv	! function output

real*8,save	:: eps=1.0D-4

dispersion_deriv=(dispersion(k+eps,omega,wd%wdepth)-dispersion(k-eps,omega,wd%wdepth))/(2.0d0*eps)
end function dispersion_deriv

end module reg_airy